/*
// Dbtest.java
//
// Created : 2002.03.11 taras
//
// Copyright (C) Dekasoft 2002
//
// License
//
// $Id:  $
//
*/
package com.dekasoft.fulcrum.query.db.test;

import junit.framework.*;
import java.sql.*;

/**
 * Test Db Evaluator core implementation
 * @author <ahref="mailto:taras@dekasoft.com.ua"> Viacheslav Tararin </a>
 */
public class DbTest extends TestCase {
// - Static -------------------------------------------------------------------
    private static final String DB_URL    = "jdbc:postgresql://192.168.1.1/spider";
    private static final String USER_NAME = "spider";
    private static final String PASSWORD  = "spider";
    private static final String DRIVER    = "org.postgresql.Driver";


// Static Constants Private ---------------------------------------------------
// .SPTM. Static ProTected Methods --------------------------------------------
    /**
    * Cteare test suite for jUnit
    */
    public static Test suite() {
        TestSuite suite = new TestSuite();
        suite.addTest(new DbTest("DataBase Error Handling test"));
        return suite;
    }
// - Instance -----------------------------------------------------------------
    private Connection m_connection = null;
// .IPBM. PuBlic Methods ------------------------------------------------------
    public DbTest(String name) {
        super(name);
    }

    protected void runTest() {
        System.out.println("testInvalidUserName()"); 
        testInvalidUserName();

        System.out.println("testInvalidSyntax()"); 
        testInvalidSyntax();

        System.out.println("testInvalidTableName()"); 
        testInvalidTableName(); 

        System.out.println("testInvalidColumnName()");
        testInvalidColumnName();

        System.out.println("testInvalidDataType()"); 
        testInvalidDataType(); 

        System.out.println("testNullValue()"); 
        testNullValue(); 

        System.out.println("testCheckFail()"); 
        testCheckFail(); 

        System.out.println("testTooLongValue()");
        testTooLongValue();

        System.out.println("testDuplicatePrimaryKey()"); 
        testDuplicatePrimaryKey(); 

        System.out.println("testDuplicateUniqueIndex()"); 
        testDuplicateUniqueIndex(); 

        System.out.println("testInsertChildWithoutParent()"); 
        testInsertChildWithoutParent(); 

        System.out.println("testUpdateChildWithoutParent()"); 
        testUpdateChildWithoutParent(); 

        System.out.println("testDeleteParentWithChildExists()"); 
        testDeleteParentWithChildExists(); 

        System.out.println("testUpdateParentWithChildExists()");
        testUpdateParentWithChildExists();
    }

    /**
     *
     */
    public void testInvalidUserName() {
        try { 
            Connection connection
                    = DriverManager.getConnection(DB_URL,
                                                  USER_NAME+"12345678",
                                                  PASSWORD);
            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testInvalidPassword() {
        try {
            Connection connection
                    = DriverManager.getConnection(DB_URL,
                                                  USER_NAME,
                                                  PASSWORD+"1234567890");
            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testInvalidSyntax() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("selec from SLAVE_TBL");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testInvalidTableName() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("select * from SLAVE_TB");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testInvalidColumnName() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("select W from SLAVE_TBL");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testInvalidDataType() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("insert into MASTER_TBL(A,B) values ('W222','W333')");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testNullValue() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("insert into MASTER_TBL(A) values ('222')");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }

        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("insert into MASTER_TBL(A,B) values (NULL,'Y')");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testCheckFail() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("insert into MASTER_TBL(A,B) values (0,'Y')");
            m_connection.commit();
            stmt.execute("insert into MASTER_TBL(A,B) values (1,'R')");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testTooLongValue() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("insert into SLAVE_TBL(C,D) values (0,'TOO LOOOOONG STRIIIIING')");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testDuplicatePrimaryKey() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("insert into MASTER_TBL(A,B) values (1,'Y')");
            m_connection.commit();

            stmt.execute("insert into MASTER_TBL(A,B) values (1,'Y')");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testDuplicateUniqueIndex() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("insert into SLAVE_TBL(C,D) values (1,'DOUBLE')");
            m_connection.commit();

            stmt.execute("insert into SLAVE_TBL(C,D) values (0,'DOUBLE')");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testInsertChildWithoutParent() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("insert into SLAVE_TBL(C,D) values (2,'TEST2')");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testUpdateChildWithoutParent() {
        try {
            Statement stmt = m_connection.createStatement();
//            stmt.execute("update SLAVE_TBL set C=2 where C=1");
            stmt.execute("update SLAVE_TBL set C=2 ");

            System.out.println("##########################");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testDeleteParentWithChildExists() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("delete from MASTER_TBL where A=1");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

    /**
     *
     */
    public void testUpdateParentWithChildExists() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("update MASTER_TBL set A=3 where A=1");

            fail("Exception expected but not raised.");
        } catch (SQLException e) {
                // TODO
        } catch (Throwable e) {
            fail("Exception:"+e);
        }
    }

// .IPTM. Protected Methods 
    protected void setUp() {
        try { 
            Class.forName(DRIVER);
        } catch (Exception e) {
            fail("Exception: "+e);
        };

            /* Chema creation */
        try {
            m_connection = DriverManager.getConnection(DB_URL, USER_NAME, PASSWORD);

            try {
                tearDown();
            } catch(Throwable e) {
                /* May be */
            }


            Statement stmt = m_connection.createStatement();
            stmt.execute(
                " create table MASTER_TBL ( "
                + "      A integer not null, "
                + "      B char(1) not null check(B in ('Y','N')), "
                + "    constraint MASTER_TBL_PK primary key (A) "
                + " ) ");
            stmt.execute(
                " create table SLAVE_TBL ( "
                + "     C integer not null, "
                + "     D varchar(10), "
                + "     constraint SLAVE_TBL_PK primary key (C) "
                + " ) ");
            stmt.execute(
                " alter table SLAVE_TBL "
                + "     add constraint SLAVE_TBL_FK1 foreign key  (C) "
                + "        references MASTER_TBL (A) ");
            stmt.execute(
                " create unique index UK_SLAVE_TBL on SLAVE_TBL(D) " );
            stmt.close();
        } catch (Throwable e) {
            fail("Exception:"+e);
        }

    };

    protected void tearDown() {
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("drop table SLAVE_TBL");
            stmt.close();
        } catch (Throwable e) {
        }
        try {
            Statement stmt = m_connection.createStatement();
            stmt.execute("drop table MASTER_TBL");
            stmt.close();
        } catch (Throwable e) {
        }
    };
}
